#include <iostream>
#include <vector>
#include <unordered_set>
#include <algorithm>
#ifdef DEBUG
#include <chrono>
#endif  // DEBUG

#define ENDL '\n'

#define NINF std::numeric_limits<int>::min()
#define INF  std::numeric_limits<int>::max()

#define MOVE_POINTER(ptr, op, label) {                                        \
    while (true) {                                                            \
        for (auto i = 0; i < targets.size(); ++i) {                           \
            auto& curr_target = targets[i];                                   \
            if (!curr_target.count(ptr)) {                                    \
                continue;                                                     \
            }                                                                 \
            if (counter[i] == 1) {                                            \
                goto label;                                                   \
            }                                                                 \
            --counter[i];                                                     \
        }                                                                     \
        op;                                                                   \
    }                                                                         \
}

namespace {
template<class T>
using vec = std::vector<T>;

template<class T>
using mat = vec<vec<T>>;

template<class T>
using uset = std::unordered_set<T>;

using tree_t = mat<int>;

// Represent a suggestion as a single 64-bit integer
// Useful for filtering unique suggestions
union suggestion_t {
    int64_t packed;
    struct {
        int32_t node;
        int32_t depth;
    };
};

using suggestions_t = uset<int64_t>;
using targets_t = vec<uset<int>>;

auto get_at_depth(tree_t const& t, uset<int>& out, int curr, int depth) {
    if (depth == 0) {
        out.emplace(curr);
        return;
    }

    for (auto& next : t[curr]) {
        get_at_depth(t, out, next, depth - 1);
    }
}

auto solve(tree_t const& tree, suggestions_t const& suggestions) {
    // Compute nodes for all suggestions
    targets_t targets(suggestions.size());

    auto i = 0;
    for (auto& packed_suggestion : suggestions) {
        suggestion_t suggestion;
        suggestion.packed = packed_suggestion;

        get_at_depth(tree, targets[i++], suggestion.node, suggestion.depth);
    }

    // Find initial left and right
    int l = INF, r = NINF;
    for (auto& target : targets) {
        auto curr_lr = std::minmax_element(target.begin(), target.end());

        l = std::min(l, *curr_lr.first);
        r = std::max(r, *curr_lr.second);
    }

    // Keep track of the amount of colored nodes per suggestion
    // At first, all nodes of all suggestions are colored
    vec<int> counter;
    counter.reserve(targets.size());

    for (auto& target : targets) {
        counter.emplace_back(target.size());
    }

    // Try to increase left
    MOVE_POINTER(l, ++l, right)

    // Try to decrease right
right:
    MOVE_POINTER(r, --r, done)

done:
    return std::make_pair(l, r);
}
}  // namespace

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);
    std::cout.tie(nullptr);

    int n;
    std::cin >> n;

    tree_t tree(n + 1);

    int p;
    for (auto i = 2; i <= n; ++i) {
        std::cin >> p;

        tree[p].emplace_back(i);
    }

    int m;
    std::cin >> m;

    suggestions_t suggestions;
    suggestions.reserve(m);

    int u, k;
    while (m--) {
        std::cin >> u >> k;

        suggestion_t suggestion;

        suggestion.node = u;
        suggestion.depth = k;

        suggestions.emplace(suggestion.packed);
    }

#ifdef DEBUG
    auto start_time = std::chrono::high_resolution_clock::now();
#endif  // DEBUG

    auto res = solve(tree, suggestions);
    std::cout << res.first << " " << res.second << ENDL;


#ifdef DEBUG
    auto end_time = std::chrono::high_resolution_clock::now();
    std::cout
        << std::chrono::duration_cast<std::chrono::milliseconds>
            (end_time - start_time).count()
        << " ms" << ENDL;
#endif  // DEBUG

    return 0;
}
