#include <iostream>
#include <cstring>
#include <vector>
#include <set>

using namespace std;

const int MAXN = 1e5 + 5;

int n, m;
set <int> graph[MAXN];
vector <pair <int, int>> edges;

bool used[MAXN];
int depth[MAXN];
int highest[MAXN];
pair <int, int> subtreeInds[MAXN];

int degActive[MAXN];
struct Component //Biconnected
{
    int n, m;
    int full;
    vector <int> nodes;

    Component()
    {
        this->n = 0;
        this->m = 0;
        this->full = 0;
    }

    int eval()
    {
        if(n==1) return 0;
        if(n==3) return 1;

        int freeEdges = min(m - (n-1), n-1);
        if(freeEdges==n-1) freeEdges -= full;

        return freeEdges;
    }
};

int compId[MAXN];
vector <Component> comps;

set <pair <int, int>> bridges;
int dfsInit(int x, int last, int level, int id)
{
    //cout << x << " -> " << id << '\n';

    used[x] = true;
    depth[x] = level;

    int maxId = id;
    subtreeInds[x].first = id;

    highest[x] = MAXN;
    int localHighest = MAXN;

    for(int y: graph[x])
    {
        if(used[y]==false)
        {
            maxId = max(maxId, dfsInit(y, x, level+1, maxId+1));
            highest[x] = min(highest[x], highest[y]);
        }
        else if(y!=last)
        {
            localHighest = min(localHighest, depth[y]);
        }
    }

    //cout << x << " -> " << highest[x] << " " << depth[x] << '\n';

    highest[x] = min(highest[x], localHighest);
    if(last!=-1 && highest[x] > depth[last])
    {
        //cout << "bridge: " << x << " " << last << '\n';
        bridges.insert({min(x, last), max(x, last)});
    }

    subtreeInds[x].second = maxId;
    return maxId;
}

void dfsComps(int x, Component &c, int id)
{
    compId[x] = id;
    used[x] = true;
    c.nodes.push_back(x);

    for(int y: graph[x])
    {
        if(used[y]==false)
            dfsComps(y, c, id);
    }
}

int main()
{

    cin >> n >> m;
    for(int i = 0;i<m;i++)
    {
        int u, v;
        cin >> u >> v;
        if(u>v) swap(u, v);

        graph[u].insert(v);
        graph[v].insert(u);
        edges.push_back({u, v});
    }

    dfsInit(1, -1, 1, 1);

    set <pair <int, int>> activeEdges;
    for(int i = 0;i<n-1;i++)
    {
        int ind;
        cin >> ind;

        degActive[edges[ind-1].first]++;
        degActive[edges[ind-1].second]++;
        activeEdges.insert(edges[ind-1]);
    }

    int activeBridges = 0;
    for(pair <int, int> e: bridges)
    {
        if(activeEdges.count(e)==true)
        {
            activeBridges++;

            degActive[e.first]--;
            degActive[e.second]--;
        }

        graph[e.first].erase(e.second);
        graph[e.second].erase(e.first);
    }

    memset(used, false, sizeof(used));
    for(int x = 1;x<=n;x++)
    {
        if(used[x]==false)
        {
            comps.push_back(Component());
            dfsComps(x, comps.back(), comps.size()-1);
        }
    }

    for(Component &c: comps)
    {
        c.n = c.nodes.size();
        for(int x: c.nodes)
        {
            if(degActive[x]==graph[x].size()) c.full++;
        }
    }
    for(pair <int, int> e: edges)
    {
        if(bridges.count(e)==false)
        {
            comps[compId[e.first]].m++;
        }
    }

    int sum = 0;
    for(Component &c: comps)
    {
        sum += c.eval();
        //cout << c.n << " -> " << c.eval() << '\n';
    }

    cout << sum + (bridges.size() - activeBridges) << '\n';
}
/*
4 4
1 2
2 3
2 4
3 4
1 3 4
*/
